# maps example
install.packages("maps")
library(maps)
map("state", interior = FALSE)
map("state", boundary = FALSE, col="gray", add = TRUE)
points(-117,34,pch=19,col="red")

# lots of points - longitude then latitude
longmin <- -140
longmax <- -50
nlong <- longmax-longmin+1
latmin <- 25
latmax <- 55
nlat <- latmax-latmin+1
points(rep(longmin:longmax,nlat),rep(latmin:latmax,nlong),pch=1,col=4)


# Neural Nets example #1
set.seed(4242)
x <- runif(84,min=0,max=2)
y <- runif(84,min=0,max=2)
labels <- ifelse( x<y, -1, 1 )
data_all <- data.frame( labels, x, y )
data_train <- data_all[1:42,]
data_test <- data_all[43:84,]
plot(data_train$x,data_train$y,col=data_train$labels+2,pch=20,cex=1) 

# Creating and training the neural network
# threshold is a numeric value specifying how soon to stop...
# If the error (the gradient) is less steep than threshold, we stop
nnet <- neuralnet(labels~x+y, 
                  data=data_train, hidden=NULL, threshold=0.01)
print(nnet)
plot(nnet) 

# to compute new data (prediction) with a neural net, use compute:
nnet.result <- compute( nnet, data_test[-1] ) # -1 gets rid of the labels
predicted_labels <- nnet.result$net.result
# these will be real numbers, with 0 the cutoff; we take the closer one
predicted_labels <- ifelse( predicted_labels<0, -1, 1 )

# compute how many were correct...
actual_labels <- data_test$labels
correct_labels <- predicted_labels == actual_labels
num_correct <- sum(correct_labels)
total_num <- length(actual_labels)
cat("The network got ",num_correct," out of ",total_num,"predictions correct.\n")

# plot the incorrect ones with a different color
correct_color <- ifelse(correct_labels,0,1)
plot(data_test$x,data_test$y,col=data_test$labels+correct_color+2,pch=20,cex=1) 
abline(0,1)



# "XOR" problem... need intermediate nodes...
# because the data is not linearly separable

set.seed(4242)
x <- runif(84,min=0,max=2)
y <- runif(84,min=0,max=2)
labels <- ifelse( (x<1 & y<1) | (x>1 & y > 1), -1, 1 )
data_all <- data.frame( labels, x, y )
data_train <- data_all[1:42,]
data_test <- data_all[43:84,]

plot(data_train$x,data_train$y,col=data_train$labels+2,pch=20,cex=1) 

#Train the neural network
# threshold is a numeric value specifying how soon to stop...
# If the changes (gradient) is less steep than threshold, we stop
nnet <- neuralnet(labels~x+y, data=data_train, hidden=NULL, threshold=0.01)
#nnet <- neuralnet(labels~x+y, data=data_train, hidden=c(3,3), threshold=0.01)
print(nnet)
plot(nnet) #Plot the neural network

# to compute new data (prediction) with a neural net, use compute:
#testing_cols <- trainingdatacbind( labels_test, x_test, y_test, colnames=c("labels","x","y") )
nnet.result <- compute( nnet, data_test[-1] ) # -1 gets rid of the labels
predicted_labels <- nnet.result$net.result
# these will be real numbers, with 0 the cutoff; we take the closer one
predicted_labels <- ifelse( predicted_labels<0, -1, 1 )
# from the data
actual_labels <- data_test$labels
correct_labels <- predicted_labels == actual_labels
num_correct <- sum(correct_labels)
total_num <- length(actual_labels)
cat("The network got ",num_correct," out of ",total_num,"predictions correct.\n")

# plot the incorrect ones with a different color
correct_color <- ifelse(correct_labels,0,1)
plot(data_test$x,data_test$y,col=data_test$labels+correct_color+2,pch=20,cex=1) 
#abline(0,1)
points(data_train$x,data_train$y,col=1,pch=1)



# example of learning a square-root function...

#Generate 50 random numbers uniformly distributed between 0 and 100
#And store them as a dataframe
traininginput <-  as.data.frame(runif(50, min=0, max=100))
trainingoutput <- sqrt(traininginput)  # the output is the sqrt of the input

#Column bind the data into one variable
trainingdata <- cbind(traininginput,trainingoutput)
colnames(trainingdata) <- c("Input","Output")

#Train the neural network
net.sqrt <- neuralnet(Output~Input,trainingdata, hidden=c(10), threshold=0.01)
print(net.sqrt)
plot(net.sqrt)

#Test the neural network on some training data
testdata <- as.data.frame((1:10)^2) #Generate some squared numbers
net.results <- compute(net.sqrt, testdata) #Run them through the neural network

#Lets see what properties net.sqrt has
ls(net.results)

#Lets see the results
print(net.results$net.result)

#Lets display a better version of the results
cleanoutput <- cbind(testdata,sqrt(testdata),
                     as.data.frame(net.results$net.result))
colnames(cleanoutput) <- c("Input","Expected Output","Neural Net Output")
print(cleanoutput)


# example of using a neural net on the iris data...
# adapted from horicky.blogspot.com/2012/06/predictive-analytics-neuralnet-bayesian.html
# library(neuralnet)
data(iris)

# set up training and testing data
testidx <- which(1:length(iris[,1])%%5 == 0) # every fifth index...
iristrain <- iris[-testidx,]  # train on 4/5 of the data (all but the testing)
iristest <- iris[testidx,]  # test on 1/5 of the data (the rest)

# nnet-specific data setup...
nnet_iristrain <- iristrain

# We need to numerically represent the categorical output
# we add three columns - one for each type
nnet_iristrain <- cbind(nnet_iristrain, iristrain$Species == 'setosa')
nnet_iristrain <- cbind(nnet_iristrain, iristrain$Species == 'versicolor')
nnet_iristrain <- cbind(nnet_iristrain, iristrain$Species == 'virginica')
# we name those three columns appropriately
names(nnet_iristrain)[6] <- 'setosa'
names(nnet_iristrain)[7] <- 'versicolor'
names(nnet_iristrain)[8] <- 'virginica'
nn <- neuralnet(setosa+versicolor+virginica ~ 
                Sepal.Length+Sepal.Width+Petal.Length+Petal.Width,
                data=nnet_iristrain, 
                hidden=c(3))  # 3 neurons in the hidden layer
plot(nn)

# prediction for the the iris data...
mypredict <- compute(nn, iristest[-5])$net.result

# A f'n to put multiple binary output to categorical output
maxidx <- function(arr) { return(which(arr == max(arr))) }
idx <- apply(mypredict, c(1), maxidx)

prediction <- c('setosa', 'versicolor', 'virginica')[idx]
table(prediction, iristest$Species)



# substitution error for the the iris data...
mypredict <- compute(nn, iristrain[-5])$net.result

# A f'n to put multiple binary output to categorical output
maxidx <- function(arr) { return(which(arr == max(arr))) }
idx <- apply(mypredict, c(1), maxidx)

prediction <- c('setosa', 'versicolor', 'virginica')[idx]
table(prediction, iristrain$Species)




# Support vector machines !

# SVM examples in R

# these first ones are adapted from the following tutorial:
# cbio.ensmp.fr/~jvert/svn/tutorials/practical/svmbasic/svmbasic_notes.pdf

set.seed(1234)
n <- 150 # number of data points
p <- 2 # dimension
sigma <- 1 # variance of the distribution
meanpos <- 0 # centre of the distribution of positive examples
meanneg <- 4 # centre of the distribution of negative examples
npos <- round(n/2) # number of positive examples
nneg <- n-npos # number of negative examples

# Generate the positive and negative examples
xpos <- matrix(rnorm(npos*p,mean=meanpos,sd=sigma),npos,p)
xneg <- matrix(rnorm(nneg*p,mean=meanneg,sd=sigma),npos,p)
x <- rbind(xpos,xneg)
# Generate the labels
y <- matrix(c(rep(1,npos),rep(-1,nneg)))
# Visualize the data
plot(x,col=ifelse(y>0,1,2),pch=19)
legend("topleft",c('Positive','Negative'),col=seq(2),pch=19,text.col=seq(2))

## Prepare a training and a test set ##
ntrain <- round(n*0.8) # number of training examples
tindex <- sample(n,ntrain) # indices of training samples
xtrain <- x[tindex,]
xtest <- x[-tindex,]
ytrain <- y[tindex]
ytest <- y[-tindex]
istrain=rep(0,n)
istrain[tindex]=1

# Visualize
plot(x,col=ifelse(y>0,1,2),pch=ifelse(istrain==1,1,19))
legend("topleft",c('Positive Train','Positive Test','Negative Train','Negative Test'),
       col=c(1,1,2,2),pch=c(1,19,1,19),text.col=c(1,1,2,2))


# load the kernlab package
library(kernlab)
# train the SVM
svp <- ksvm(xtrain,ytrain,type="C-svc",kernel='vanilladot',C=100,scaled=c())

# General summary
svp
# Attributes that you can access
attributes(svp)
# For example, the support vectors
alpha(svp)
alphaindex(svp)
b(svp)
# Use the built-in function to pretty-plot the classifier
plot(svp,data=xtrain)


# Predict labels on test
ypred = predict(svp,xtest)
table(ytest,ypred)
# Compute accuracy
acc <- sum(ypred==ytest)/length(ytest)
cat("Accuracy: ", acc)
# Compute at the prediction scores
ypredscore = predict(svp,xtest,type="decision")





# try one with non-linearly-separable data...
set.seed(4242)
n <- 150 # number of data points
p <- 2 # dimension
sigma <- 1 # variance of the distribution
meanpos <- 0 # centre of the distribution of positive examples
meanneg <- 2.5 # centre of the distribution of negative examples
npos <- round(n/2) # number of positive examples
nneg <- n-npos # number of negative examples

# Generate the positive and negative examples
xpos <- matrix(rnorm(npos*p,mean=meanpos,sd=sigma),npos,p)
xneg <- matrix(rnorm(nneg*p,mean=meanneg,sd=sigma),npos,p)
x <- rbind(xpos,xneg)
# Generate the labels
y <- matrix(c(rep(1,npos),rep(-1,nneg)))
# Visualize the data
plot(x,col=ifelse(y>0,1,2),pch=19)
legend("topleft",c('Positive','Negative'),col=seq(2),pch=19,text.col=seq(2))

## Prepare a training and a test set ##
ntrain <- round(n*0.8) # number of training examples
tindex <- sample(n,ntrain) # indices of training samples
xtrain <- x[tindex,]
xtest <- x[-tindex,]
ytrain <- y[tindex]
ytest <- y[-tindex]
istrain=rep(0,n)
istrain[tindex]=1

# Visualize
plot(x,col=ifelse(y>0,1,2),pch=ifelse(istrain==1,1,19))
legend("topleft",c('Positive Train','Positive Test','Negative Train','Negative Test'),
       col=c(1,1,2,2),pch=c(1,19,1,19),text.col=c(1,1,2,2))


# load the kernlab package
library(kernlab)
# train the SVM
svp <- ksvm(xtrain,ytrain,type="C-svc",kernel='vanilladot',C=100,scaled=c())

# General summary
svp
# Attributes that you can access
attributes(svp)
# For example, the support vectors
alpha(svp)
alphaindex(svp)
b(svp)
# Use the built-in function to pretty-plot the classifier
plot(svp,data=xtrain)


# Predict labels on test
ypred = predict(svp,xtest)
table(ytest,ypred)
# Compute accuracy
acc <- sum(ypred==ytest)/length(ytest)
cat("Accuracy: ",acc)

# could plot the mis-classified ones... from the test data...



# try one with truly non-linearly-separable data...
set.seed(4242)
n <- 150 # number of data points
p <- 2 # dimension
sigma <- 1 # variance of the distribution
meanpos1 <- c(0,0) # centre of the distribution of positive examples
meanneg1 <- c(5,0) # centre of the distribution of negative examples
meanpos2 <- c(5,5) # centre of the distribution of positive examples
meanneg2 <- c(0,5) # centre of the distribution of negative examples
npos <- round(n/2) # number of positive examples
nneg <- n-npos # number of negative examples

# Generate the positive and negative examples
xpos1 <- matrix(rnorm(npos*p,mean=meanpos1,sd=sigma),npos,p)
xneg1 <- matrix(rnorm(nneg*p,mean=meanneg1,sd=sigma),nneg,p)
xpos2 <- matrix(rnorm(npos*p,mean=meanpos2,sd=sigma),npos,p)
xneg2 <- matrix(rnorm(nneg*p,mean=meanneg2,sd=sigma),nneg,p)
x <- rbind(xpos1,xpos2,xneg1,xneg2)
# adjust the amounts
npos <- 2*npos
nneg <- 2*nneg
n <- 2*n
# Generate the labels
y <- matrix(c(rep(1,npos),rep(-1,nneg)))
# Visualize the data
plot(x,col=ifelse(y>0,1,2),pch=19)
legend("topleft",c('Positive','Negative'),col=seq(2),pch=19,text.col=seq(2))

## Prepare a training and a test set ##
ntrain <- round(n*0.8) # number of training examples
tindex <- sample(n,ntrain) # indices of training samples
xtrain <- x[tindex,]
xtest <- x[-tindex,]
ytrain <- y[tindex]
ytest <- y[-tindex]
istrain=rep(0,n)
istrain[tindex]=1

# Visualize
plot(x,col=ifelse(y>0,1,2),pch=ifelse(istrain==1,1,19))
legend("topleft",c('Positive Train','Positive Test','Negative Train','Negative Test'),
       col=c(1,1,2,2),pch=c(1,19,1,19),text.col=c(1,1,2,2))


# load the kernlab package
library(kernlab)
# train the SVM
svp <- ksvm(xtrain,ytrain,type="C-svc",kernel='vanilladot',C=1)
#svp <- ksvm(xtrain,ytrain,type="C-svc",kernel='rbf',kpar=list(sigma=1),C=1)

# General summary
svp
# Attributes that you can access
attributes(svp)
# For example, the support vectors
alpha(svp)
alphaindex(svp)
b(svp)
# Use the built-in function to pretty-plot the classifier
plot(svp,data=xtrain)


# Predict labels on test
ypred = predict(svp,xtest)
table(ytest,ypred)
# Compute accuracy
acc <- sum(ypred==ytest)/length(ytest)
cat("Accuracy: ",acc)



# parameters!
# Train a nonlinear SVM with automatic selection of sigma by heuristic
svp <- ksvm(xtrain,ytrain,type="C-svc",kernel='rbf',C=1)
# Visualize it
plot(svp,data=xtrain)

#?kernels


# on iris data....
# first from Rick Ho

# then using multiclass... for iris

data(iris)
n <- nrow(iris)
testIndex <- sample(1:n, floor(n/3))
iristrain <- iris[-testIndex, ]
iristest <- iris[testIndex, ]

# auto-detect the best gamma...
svmmod <- ksvm(Species ~ ., data = iristrain, kernel = "rbfdot", C = 100)
type.pred <- predict(svmmod, iristest[, -5])
table(predicted = type.pred, true = iristest[, 5])

# plot... # can't plot more than binary!
#plot(svmmod,data=iristrain)


# spam example...

data(spam)
n <- nrow(spam)
testIndex <- sample(1:n, floor(n/3))
spamtrain <- spam[-testIndex, ]
spamtest <- spam[testIndex, ]

# auto-detect the best gamma...
svmmod <- ksvm(type ~ ., data = spamtrain, kernel = "rbfdot", C = 5)
type.pred <- predict(svmmod, spamtest[, -58])
table(predicted = type.pred, true = spamtest[, 58])

# cool - check out that data! ?spam


# k-NN methods...
# are nice, because they're simple!
library(class)
data(iris)
n <- nrow(iris)
testIndex <- sample(1:n, floor(n/3))
iristrain <- iris[-testIndex, ]
iristest <- iris[testIndex, ]
train_input <- as.matrix(iristrain[,-5])
train_output <- as.vector(iristrain[,5])
test_input <- as.matrix(iristest[,-5])

prediction <- knn(train_input, test_input, train_output, k=5)
table(prediction, iristest$Species)


  

